<?php
/*
Plugin Name: Amazon Search
Plugin URI: http://wordpress.org/extend/plugins/amazon-search/
Description: Adds a search widget to allow searching of any international Amazon site and displays
the results right in your blog.  Requires WordPress 2.2+.
Author: Andy Maloney
Version: 1.0.1
Author URI: http://imol.gotdns.com/
*/


if (!class_exists('AMZ_Search'))
{
	class AMZ_Search
	{
		var $_version = '1.0.1';
		
		var $_adminOptionsName = 'AMZ_SearchAdminOptions';
		var $_db_table;
		
		var $_cleanData;
		var $_pluginURL;


		
		function AMZ_Search()
		{ 
			global $wpdb;

			$this->_db_table = $wpdb->prefix . "amz_servers";
			$this->_pluginURL = get_option('siteurl') . '/wp-content/plugins/amazon-search';
		} 
		
		function init()
		{ 
			$this->getAdminOptions();
		}		
				
		function addHeaderCode()
		{
			echo '<link type="text/css" rel="stylesheet" href="' . get_bloginfo('wpurl') . '/wp-content/plugins/amazon-search/css/default.css" />' . "\n";
		
			// check for CSS file specific to this theme
			$template = get_option('template');
			
			if ( $template !== "default" )
			{
				$theme_css = getcwd() . '/wp-content/plugins/amazon-search/css/' . $template . '.css';
				
				if ( is_readable( $theme_css ) )
					echo '<link type="text/css" rel="stylesheet" href="' . get_bloginfo('wpurl') . '/wp-content/plugins/amazon-search/css/' . $template . '.css" />' . "\n";
			}			
		}

		function db_install()
		{
			global $wpdb;
		
			$table_name = $this->_db_table;
			if($wpdb->get_var("SHOW TABLES LIKE '$table_name'") != $table_name)
			{
				$sql = "CREATE TABLE {$table_name} (
					  id enum('com','ca','uk','fr','de','jp') NOT NULL default 'com',
					  server enum('amazon.com','amazon.ca','amazon.co.uk','amazon.fr','amazon.de','amazon.co.jp') NOT NULL default 'amazon.com',
					  assoc_server enum('affiliate-program.amazon.com','associates.amazon.ca','affiliate-program.amazon.co.uk','partenaires.amazon.fr','partnernet.amazon.de','affiliate.amazon.co.jp') NOT NULL default 'affiliate-program.amazon.com',
					  default_tag enum('imol-20','imol0a-20','imol-21','imol08-21','imol05-21','imol-22') NOT NULL default 'imol-20',
					  user_tag varchar(32) default NULL,
					  PRIMARY KEY  (id),
					  UNIQUE KEY server (server)
					) PACK_KEYS=1
					";
		
				require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
				dbDelta($sql);
		
				$_DATA[] = "INSERT IGNORE INTO {$table_name} VALUES ('com', 'amazon.com', 'affiliate-program.amazon.com', 'imol-20', '')";
				$_DATA[] = "INSERT IGNORE INTO {$table_name} VALUES ('ca', 'amazon.ca', 'associates.amazon.ca', 'imol0a-20', '')";
				$_DATA[] = "INSERT IGNORE INTO {$table_name} VALUES ('uk', 'amazon.co.uk', 'affiliate-program.amazon.co.uk', 'imol-21', '')";
				$_DATA[] = "INSERT IGNORE INTO {$table_name} VALUES ('fr', 'amazon.fr', 'partenaires.amazon.fr', 'imol08-21', '')";
				$_DATA[] = "INSERT IGNORE INTO {$table_name} VALUES ('de', 'amazon.de', 'partnernet.amazon.de', 'imol05-21', '')";
				$_DATA[] = "INSERT IGNORE INTO {$table_name} VALUES ('jp', 'amazon.co.jp', 'affiliate.amazon.co.jp', 'imol-22', '')";
		
		    	foreach ($_DATA as $insert)
		    	{
					$results = $wpdb->query( $insert );
				}
				
				add_option("AMZ_db_version", $this->_version);
		   }
		}

		function getAdminOptions()
		{			
			$adminOptions = array(
				'contribute' => '4', 
				'default_category' => 'Blended',  
				'default_search' => 'WordPress',  
				'default_server' => 'com',
				'image_size' => 'Medium',
				'search_count' => '0',
				'xsl' => $this->_pluginURL . "/xsl/item-search.xsl",
				);
				
			$savedOptions = get_option($this->_adminOptionsName);
			
			$dirty = false;
			
			if (!empty($savedOptions))
			{ 
				foreach ($savedOptions as $key => $option)
				{
					if ( $adminOptions[$key] != $option )
					{
						$dirty = true;
						
						$adminOptions[$key] = $option;
					}
				}
			}             
		
			if ( $dirty )
				update_option($this->_adminOptionsName, $adminOptions);
			
			return $adminOptions; 
		} 
		
		function printSearchBox()
		{			
			$this->storeCleanRequestData();
			$clean = $this->_cleanData;
			
			$display = '<div class="amz-search-widget">';
			$display .= '<form method="get" action="">';	
			$display .= '<span class=server>';
			$display .= 'amazon. ';
				
			$display .= $this->_buildServerOptionList( 'a_server', $clean['a_server'] );
			$display .= '</span>';
			
			$keyword = ( $clean['no_keyword'] ) ? '' : $clean['keyword'];
			$display .= '<p><input type="text" name="field-keywords" size="17" value="' . $keyword . '" class="keyword-input"/>';
		
			$display .= '<input type="image" value="Go" name="amz_search" '
						. 'src="' . $this->_pluginURL . '/images/amazon-go.gif" '
						. ' class="go"/>';
			$display .= '</form>';
			$display .= '<input type="hidden" name="category" value="' . $clean['category'] . '"/>';
			$display .= '</div>';
				
			echo $display;
		}

		function storeCleanRequestData()
		{	
			$_AMZ_PREFS = $this->getAdminOptions();
		
			// default and default default server
			if ( !empty( $_REQUEST['a_server'] ) )
				$a_server = substr( $_REQUEST['a_server'], 0, 3 );
			else
				$a_server = $_AMZ_PREFS['default_server'];
			
			$clean['a_server'] = $this->_validServer( $a_server );
			
			// default and default default category
			if ( !empty( $_REQUEST['category'] ) )
				$category = $_REQUEST['category'];
			else
				$category = $_AMZ_PREFS['default_category'];
			
			$clean['category'] = $this->_validCategory( $category );
			
			// default and default default keyword
			$clean['no_keyword'] = true;
			$keyword = '';
			
			if ( !empty( $_REQUEST['field-keywords'] ) )
			{
				$keyword = $_REQUEST['field-keywords'];
				
				$clean['no_keyword'] = false;
			}
						
			$clean['keyword'] = $this->_validKeyword( $keyword );
			
			if ( empty( $clean['keyword'] ) )
			{
				if ( !empty( $_AMZ_PREFS['default_search']) )
					$clean['keyword'] = $this->_validKeyword( $_AMZ_PREFS['default_search'] );
			
				if ( empty( $clean['keyword'] ) )
					$clean['keyword'] = 'Kahlil Gibran';
			
				$clean['no_keyword'] = true;
			}
			
			// which page of results
			if ( !empty( $_REQUEST['ItemPage'] ) )
			{
				$ItemPage = $_REQUEST['ItemPage'];
				settype( $ItemPage, 'integer' );
			}
			else
			{
				$ItemPage = 1;
			}
				
			$clean['item_page'] = $ItemPage;
			
			if ( $clean['a_server'] == 'com' )
				$clean['xml_server'] = 'us';
			else
				$clean['xml_server'] = $clean['a_server'];
									
			$this->_cleanData = $clean;
		}

		function getSearchResults()
		{
			global $wpdb;
	
			$_AMZ_PREFS = $this->getAdminOptions();
			$clean = $this->_cleanData;
	
			// Get the server info
			$sql = 'SELECT * FROM ' . $this->_db_table . ' WHERE id="' . $clean['a_server'] .'"';
			$server_info = $wpdb->get_row($sql, ARRAY_A);
		
			$amazon_server = $server_info['server'];
			$ass_id = $server_info['user_tag'];
			$flag = $this->_pluginURL . '/images/flag_' . $server_info['id'] . '.gif';
			
			if ( $ass_id == NULL )
			{
				$ass_id = $server_info['default_tag'];
			}
			else if ( $_AMZ_PREFS['contribute'] > 0 )
			{
				$_AMZ_PREFS['search_count'] += 1;
			
				if ( $_AMZ_PREFS['search_count'] >= $_AMZ_PREFS['contribute'] )
				{
					$ass_id = $server_info['default_tag'];
					$_AMZ_PREFS['search_count'] = 0;
				}
				
				// Update the search count
				update_option($this->_adminOptionsName, $_AMZ_PREFS); 
			}
	
			$display = '<div class="amz-search">';
			$display .= '<form method="get" action="' . $_SERVER['SCRIPT_NAME'] . '">';
			$display .= '<span class="server">';
			$display .= 'Search amazon. ';
			$display .= $this->_buildServerOptionList( 'a_server', $clean['a_server'] );
			$display .= $this->_buildCategoryOptionList( 'category', $clean['category'] );
			$display .= '</span>';
			
			$display .= '<p><input type=text name="field-keywords" size=30 value="' . $clean['keyword'] . '"  class="keyword-input"/>';
			$display .= '<input type="image" value="Go" name="amz_search" '
					. 'src="' . $this->_pluginURL . '/images/amazon-go.gif"'
					. ' class="go"/>';
			
			$display .= '</form>';
			
			$display .= '<hr/>';
			
			if ( $clean['no_keyword'] )
				$display .= "<p>You did not specify a valid search term, so here is a search for '{$clean['keyword']}'.<br/>";
				
			$amazon_search_url = "http://xml-{$clean['xml_server']}.amznxslt.com/onca/xml?";
			
			// First the amazon vars...
			$amazon_search_url .= "Service=AWSECommerceService&SubscriptionId=1M17J4TZNSDYTRSTAZ82";
			$amazon_search_url .= "&ResponseGroup=Medium";
			$amazon_search_url .= "&AssociateTag=$ass_id";
			$amazon_search_url .= "&Operation=ItemSearch";
			$amazon_search_url .= "&ItemPage={$clean['item_page']}";
			$amazon_search_url .= "&SearchIndex={$clean['category']}";
			if ( $clean['category'] != 'Blended' )
			{
				$amazon_search_url .= "&Sort=salesrank";
			}
			$amazon_search_url .= "&Keywords=" . urlencode( $clean['keyword'] );
			$amazon_search_url .= "&Style={$_AMZ_PREFS['xsl']}";
			$amazon_search_url .= "&Version=2007-10-29";
			
			// ...then ours
			$amazon_search_url .= "&ServerName=$amazon_server";
			$amazon_search_url .= "&a_server={$clean['a_server']}";
			$amazon_search_url .= "&Flag=$flag";
			$amazon_search_url .= "&ImgSize={$_AMZ_PREFS['image_size']}";
			$amazon_search_url .= "&keyword=" . urlencode( urlencode( $clean['keyword'] ) );		// [sic]
			
			// $display .= 'Search URL:    ' . $amazon_search_url . '<br/>';
			
			// grab search results from amazon
			$ch = curl_init( $amazon_search_url );
			
			curl_setopt( $ch, CURLOPT_HEADER, 0 );
			curl_setopt( $ch, CURLOPT_RETURNTRANSFER, 1 );
			
			$results = curl_exec( $ch );
			
			if ( $err = curl_error( $ch ) )
			{
				$display .= $err . '<div class="error">Please report this to the site administrator.<div/>';
			}
			
			//$display .= 'Download speed =' . curl_getInfo( $ch, CURLINFO_SPEED_DOWNLOAD ) . '<br/>';
				 
			curl_close( $ch );
			
			// IF we are using ISO-8859-1
			//	THEN convert from UTF-8 so that the pound sign and German chars display properly
			if ( $LANG_CHARSET == 'iso-8859-1' )
			{
				$results = utf8_decode( $results );
			}
			
			$display .= $results;
			$display .= '</div>';
			
			return $display;
		}

		function printAdminPage()
		{			
			if (!function_exists( 'curl_init' ) )
			{
				echo '<div class="error"><b>Your HTTP server does not support curl.  The Amazon Search plugin will not work.</b></div>';
				return;
			}

			global $wpdb;

			$display = '';
			
			$_AMZ_PREFS = $this->getAdminOptions();

			// Check for and save updated prefs
			if ( isset( $_POST['Save'] ) )
			{
				check_admin_referer('amz_search-admin_options');
				
				$new_prefs = $_POST['new_prefs'];
				
				foreach ( $new_prefs as $pref => $value )
				{
					if (isset($new_prefs[$pref]))
					{
						$_AMZ_PREFS[$pref] = $new_prefs[$pref];
					}	
				}
							
				update_option($this->_adminOptionsName, $_AMZ_PREFS);

				$results = $wpdb->get_results('SELECT * FROM ' . $this->_db_table, ARRAY_A);
				foreach ($results as $server_info)
				{
					$associate_id = trim( $_POST['usertag'][$server_info['id']] );
					$sql = 'UPDATE ' . $this->_db_table . " SET user_tag='$associate_id' WHERE id='{$server_info['id']}'";
					$wpdb->query($sql);
				}
				
				$display .= '<div class="updated"><p><strong>Settings Updated.</strong></p></div>';
			}

			$display .= '<form method="POST" action="' . $_SERVER["REQUEST_URI"] . '">';
			
			$display .= $this->_nonce_field('amz_search-admin_options');

			// Submit
			$display .= '<div style="border:#C0C0C0 dotted 1px; padding: 5px; text-align: right;">
				Amazon Search v' . $this->_version . ' by <a href="http://imol.gotdns.com">Andy Maloney</a> <input type=submit name=Save value="Save"/>
				</div>';
						
			// Amazon Associate prefs
			$display .= '<div style="border:#C0C0C0 dotted 1px; padding: 10px;">
				<u>Associate IDs</u><br/>
				<table cellspacing=5 width="100%">
				<tr style="font-weight: bold;">
				<td align=center>Default</td>
				<td colspan="2">Amazon Server</td>
				<td>Associate ID</td>
				<td>Web Site</td>
				</tr>';
			
			$results = $wpdb->get_results('SELECT * FROM ' . $this->_db_table, ARRAY_A);
			foreach ($results as $server_info)
			{
				$flag_url = $this->_pluginURL . '/images/flag_' . $server_info['id'] . '.gif';
				$alt_flag = $server_info['id'] . ' flag';
			
				$display .= '<tr>';
				
				$display .= '<td width=70 align=center>';
				$display .= '<input type=radio name="new_prefs[default_server]"';
				$display .= ($_AMZ_PREFS['default_server'] == $server_info['id'] ? ' checked' : '' );
				$display .= ' value="' . $server_info['id'] . '"/>';
				$display .= '</td>';
				
				$display .= '<td width=30><img src="' . $flag_url . '" width=24 height=13 alt="' . $alt_flag . '"/></td>';
				
				$display .= '<td width=100>' . $server_info['server'] . '</td>';
				
				$display .= '<td><input type=text name="usertag[' . $server_info['id'] . ']"';
				$display .= ' size=24 value="' . $server_info['user_tag'] . '"/></td>';
			
				$display .= '<td><a href="https://' . $server_info['assoc_server'] . '" target=_blank>';
				$display .= $server_info['assoc_server'] . '</a></td>';
				
				$display .= '</tr>';
			}
			
			$display .= '</table></div>';
			
			// Contribute to Andy prefs
			$display .= '<div style="border:#C0C0C0 dotted 1px; padding: 10px 10px 0px 10px;">
				<u>Development Contribution</u><p>
				Please help support development of this plugin by using Andy\'s associate ids for some of your searches.<br/>
				<p>Use Andy\'s associate id every
				<select name="new_prefs[contribute]">';
			
			$values = array( 0, 2, 3, 4, 5, 6, 7, 8, 9, 10, 25, 50 );
			
			foreach ( $values as $value )
			{
				$display .= '<option' . ($_AMZ_PREFS['contribute'] == $value ? ' selected' : '' ) . ">$value</option>";
			}
			
			$display .= '</select> searches.</div>';
			
			// Search prefs
			$display .= '<div style="border:#C0C0C0 dotted 1px; padding: 10px;">
				<u>Searching</u><br/>
				<table cellspacing=8 width="100%">
				<tr><td align=right>Default Category</td>
				<td>';
			
			$display .= $this->_buildCategoryOptionList('new_prefs[default_category]', $_AMZ_PREFS['default_category']);
			$display .= '</td>';
			$display .= '</tr>';
			
			$display .= '<tr><td align=right>Default Search Term</td>';
			$display .= '<td><input type=text name="new_prefs[default_search]" size=64 value="' . $_AMZ_PREFS['default_search'] . '"/></td>';
			$display .= '</tr>';
			
			$display .= '<tr><td align=right>XSL file [full URL]</td>';
			$display .= '<td><input type=text name="new_prefs[xsl]" size=64 value="' . $_AMZ_PREFS['xsl'] . '"/></td>';
			$display .= '</tr>';
			
			$display .= '<tr><td align=right>Image Size</td>';
			$display .= '<td><select name="new_prefs[image_size]">';
			$display .= '<option' . ($_AMZ_PREFS['image_size'] == 'None' ? ' selected' : '' ) . ">None</option>";
			$display .= '<option' . ($_AMZ_PREFS['image_size'] == 'Small' ? ' selected' : '' ) . ">Small</option>";
			$display .= '<option' . ($_AMZ_PREFS['image_size'] == 'Medium' ? ' selected' : '' ) . ">Medium</option>";
			$display .= '<option' . ($_AMZ_PREFS['image_size'] == 'Large' ? ' selected' : '' ) . ">Large</option>";
			$display .= '</select></td>';
			$display .= '</tr>';
			
			$display .= '</table></div>';
			
			// Submit
			$display .= '<div style="border:#C0C0C0 dotted 1px; padding: 5px; text-align: right;">
				[<a href="http://imol.gotdns.com">Andy\'s site</a>] <input type=submit name=Save value="Save"/>
				</div>
				</form>';

			echo $display;
		}



		function _buildServerOptionList( $name, $selected )
		{
			$options = '<select name="' . $name . '" class="server-select">';
			$options .= '<option value="com" ' . ($selected == 'com' ? SELECTED : '') . '>com&nbsp;&nbsp;</option>';
			$options .= '<option value="ca" ' . ($selected == 'ca' ? SELECTED : '') . '>ca</option>';
			$options .= '<option value="uk" ' . ($selected == 'uk' ? SELECTED : '') . '>uk</option>';
			$options .= '<option value="fr" ' . ($selected == 'fr' ? SELECTED : '') . '>fr</option>';
			$options .= '<option value="de" ' . ($selected == 'de' ? SELECTED : '') . '>de</option>';
			$options .= '<option value="jp" ' . ($selected == 'jp' ? SELECTED : '') . '>jp</option>';
			$options .= '</select>';
			
			return $options;
		}
		
		function _buildCategoryOptionList( $name, $selected )
		{
			$category = '<select name="' . $name . '" class="category-select">';
			$category .= '<option ' . ($selected == 'Blended' ? 'SELECTED ' : '' ) . 'value="Blended">All</option>';
			$category .= '<option ' . ($selected == 'Books' ? 'SELECTED ' : '' ) . 'value="Books">Books</option>';
			$category .= '<option ' . ($selected == 'DVD' ? 'SELECTED ' : '' ) . 'value="DVD">DVDs</option>';
			$category .= '<option ' . ($selected == 'Music' ? 'SELECTED ' : '' ) . 'value="Music">Music</option>';
			$category .= '<option ' . ($selected == 'Software' ? 'SELECTED ' : '' ) . 'value="Software">Software&nbsp;&nbsp;</option>';
			$category .= '</select>';
			
			return $category;
		}
		
		// Make sure we have a valid server
		function _validServer( $a_server )
		{
			switch ( $a_server )
			{
				case 'com':
				case 'ca':
				case 'uk':
				case 'fr':
				case 'de':
				case 'jp':
					$clean_server = $a_server;
					break;
					
				case '':
					$clean_server = 'com';
					break;
					
				default:
					die( 'Invalid search data' );
			}
			
			return( $clean_server );
		}
		
		// Make sure we have a valid category
		function _validCategory( $category )
		{
			switch ( $category )
			{
				case 'Blended':
				case 'Books':
				case 'DVD':
				case 'Music':
				case 'Software':
					$clean_category = $category;
					break;
			
				case '':
					$clean_category = 'Blended';
					break;	
					
				default:
					die( 'Invalid search data' );
			}
			
			return( $clean_category );
		}
		
		// Clean up and return a keyword
		function _validKeyword( $keyword )
		{	
			$p = strip_tags( $keyword );
			$p = trim( $p );
			$p = attribute_escape( $p );
			
			return( $p );
		}
		
		// my own nonce functions because I don't want to echo the stuff immediately...
		function _nonce_field($action = -1, $name = "_wpnonce", $referer = true)
		{
			$name = attribute_escape($name);
			$ret = '<input type="hidden" name="' . $name . '" value="' . wp_create_nonce($action) . '" />';
			if ( $referer )
				$ret .= $this->_referer_field();
				
			return $ret;
		}
		
		function _referer_field()
		{
			$ref = attribute_escape($_SERVER['REQUEST_URI']);
			$ret = '<input type="hidden" name="_wp_http_referer" value="'. $ref . '" />';
			if ( wp_get_original_referer() )
			{
				$original_ref = attribute_escape(stripslashes(wp_get_original_referer()));
				$ret .= '<input type="hidden" name="_wp_original_http_referer" value="'. $original_ref . '" />';
			}
			
			return $ret;
		}
	} 
}

if (class_exists('AMZ_Search'))
{
	$gAMZ_search = new AMZ_Search();
} 


// Initialize the admin panel
function AMZ_Search_adminPage()
{
	global $gAMZ_search;
	if (!isset($gAMZ_search))
		return;
	
	if (function_exists('add_options_page'))
		add_options_page('Amazon Search', 'Amazon Search', 9, basename(__FILE__), array(&$gAMZ_search, 'printAdminPage'));
}	

// Initialize the widget
function AMZ_Search_initWidgets()
{		
	function widget_amz_search($args)
	{
		global $gAMZ_search;
		
		
		extract($args);
		echo $before_widget;
		$gAMZ_search->printSearchBox();
		echo $after_widget;
	}

	register_sidebar_widget('Amazon Search', 'widget_amz_search');
}

function AMZ_Search_results()
{	
	global $gAMZ_search;
	
	$gAMZ_search->storeCleanRequestData();

	function AMZ_search()
	{
		global $gAMZ_search;
		
		return $gAMZ_search->getSearchResults();
	}

	include('wp-content/plugins/amazon-search/templates/search.php');
	exit;
}


// Handle all our main registrations and actions
if (isset($gAMZ_search))
{
	register_activation_hook( __FILE__, array(&$gAMZ_search, 'db_install') );
		
	add_action('wp_head', array(&$gAMZ_search, 'addHeaderCode'), 1);
	
	add_action('admin_menu', 'AMZ_Search_adminPage');
	
	add_action('activate_amz-search/amz-search.php', array(&$gAMZ_search, 'init'));

	if (function_exists( 'curl_init' ) )
	{
		if ( isset($_REQUEST['amz_search']) )
			add_action('template_redirect', 'AMZ_Search_results');

		add_action('widgets_init','AMZ_Search_initWidgets');
	}
}

?>